%Use this function to create figure 3, and generate the data for all of
%the heat maps.
%
%To run it, you will need to have all of the .mat files generated by R in
%the same directory.  It is a good idea to open every one of those
%documents (preferably when you have no other variables in matlab), and
%then resave them in Matlab (this reduces the file size and speeds up them
%opening later).


if(~exist('SYNT'))
    SYNT=0;
end


%thename is the first part of the names of your .mat file.  Right now, the
%file needs to have a name of [thename, (the r_diff), '_Ac', (the r_disp)].
thename = '20170110Final_diagram_Ad'

%This is the name of your figure.
figName = 'FinalBigDiagram2'

ALTNAME=1;

load('20170110Final_diagram_Ad1_Ac1.mat')
SYNT=0;
ANALYZE=1;  %set ANALYZE to 0, it just makes the figure, without running 
            %analysis to make the mean field figures.
SHAKEITUP=1;  %this should always be 0.

%theList and theList2 is a list of all of the possible r_diff's and
%r_disp's.  Make sure that these match the output of your R file (or are a
%subset of them), otherwise the simulation will not work.
theList=[1:2:15];
theList2=[1:2:25];

%ADJ determines how far apart your communities should be in Figure 2B.  If
%ADJ=1, they will touch each other.  Right now it is set to 1.05, so that
%there is a tiny band (5% of the size of the community's box) around each
%community.
ADJ=1.05;

%This is the matrix that all of the data will be put into.  You will
%eventually plot this to generate Fig. 2B.
bigGrid=zeros(length(theList)*SIZE*ADJ-SIZE*(ADJ-1),...
    length(theList2)*SIZE*ADJ-SIZE*(ADJ-1));

%Each forHeat data is a matrix of values, each value corresponding to a
%given community.  The (i,j)'th entry corresponds to an community with the
%i'th r_disp (microbe dispersal distance) and the j'th r_diff (compound
%diffusion distance).  
%
%I explain what each is used for below (as I'm filling them in).
%forHeatFDiff=zeros(length(theList),length(theList2));
forHeatBirthX=zeros(length(theList),length(theList2));
forHeatBirth0=zeros(length(theList),length(theList2));
forHeatBirthXs=zeros(length(theList),length(theList2));
forHeatBirth0s=zeros(length(theList),length(theList2));
forHeatBirthMin=zeros(length(theList),length(theList2));
forHeatFDCovAAX=zeros(length(theList),length(theList2));
forHeatFDCovAA0=zeros(length(theList),length(theList2));
forHeatCorr=zeros(length(theList),length(theList2));
forHeatCorrAlt=zeros(length(theList),length(theList2));

%theDiff will be a matrix, where each entry is r_diff (i.e., compound 
%diffusion radius), for the corresponding
%community.  Thus, the (i,j)'th entry of theDiff will give r_diff for the
%(i,j)'th entry of each forHeat matrix.
theDiff=zeros(length(theList),length(theList2));

%theDisp is analogous, except for r_disp (i.e., microbe dispersal radius).
theDisp=zeros(length(theList),length(theList2));

%This for loop goes through every file number listed in theList.  
for ii=1:length(theList)
    for jj=1:length(theList2)
        
        %i is the actual file number used.
        i=theList(ii);
        j=theList2(jj);
        
        
        [i j] %this just tells me where we are in the program.
        
        %here we load the data.  It is important that names are consistent.
        if(ALTNAME==0)
            load([thename,num2str(i),'_Ap',num2str(j)])
        else
            load([thename,num2str(j),'_Ac',num2str(i)])
        end
        
        %If you have set SYNT=0, or not set a SYNT value, then it will
        %analyze data from N1 (the end of the simulation).  If SYNT is
        %non-zero, then it will look at data from N1a (the moment before
        %the cheaters were introduced).
        if(SYNT)
            N=N1a;
        else
            N=N1;
        end
        
        %Here I save the community snapshot data into bigGrid.  
        bigGrid(ADJ*SIZE*(ndist-1)/2+1:ADJ*SIZE*(ndist-1)/2+SIZE,...
            ADJ*SIZE*(n2dist-1)/2+1:ADJ*SIZE*(n2dist-1)/2+SIZE)=N;
        
        %If you want to generate all of the forHeat matrices, then set
        %ANALYZE to any non-zero number.  It takes a long time, however, so
        %I give you the option of not doing it.
        if(ANALYZE)
            
            %If the cheater completely crashed the community, then I set
            %every forHeat value to NaN.  To save time, I check if the
            %community is wiped out here.  Otherwise, I do the analysis.
            if(mean(N)>0)
                
                NN=N1;
                
                %analyze_record_update4 is a subroutine that analyzes a
                %community named NN, and generates all of the statistics we
                %need.
                analyze_record_update4
                
                %here1 and here2 are the 
                here1=(ndist-1)/2+1;
                here2=(n2dist-1)/2+1;
                theDisp(here1,here2)=theList(ii);
                theDiff(here1,here2)=theList2(jj);
                               
                
                %Use for Fig. S4 and S5.
                %forHeatBirth0 tells the average resource uptake for the
                %cheater (averaged across all sites).  forHeatBirthX is the
                %average resource uptake for the cross-feeder (i.e., the
                %average for each, averaged across species).  If there is
                %an 's' at the end, this value is calculated after the
                %community has been randomly rearranged.  forHeatBirthMin
                %is what the cheater's birth rate would be if it was
                %limited by the rarer compound in the entire community.
                %For Figure S4, use:
                %  forHeatBirth0s./forHeatBirth0-1.
                %For Figure S5, use:
                %  forHeatBirth0./forHeatBirthMin.
                forHeatBirth0(here1,here2)=birth0;
                forHeatBirthX(here1,here2)=(birth1+birth2)/2;
                forHeatBirthXs(here1,here2)=(birth1shake+birth2shake)/2;
                forHeatBirth0s(here1,here2)=birth0shake;
                forHeatBirthMin(here1,here2)=min(birth1,birth2);
                
                %Use for Fig. S3
                %This calculates how much more likely a microbe is to be in
                %high-resource areas (specifically, if you picked up a
                %microbe and dropped it in a random location, this is how
                %much the density of the resource it needs will change on
                %average).  AAX is for the cross-feeders, AA0 is for the
                %cheater.
                %Use forHeatFDCovAAX for part A, use forHeatFDCovAA0 for
                %part B, and use (forHeatFDCovAAX-forHeatFDCovAA0) for part
                %C.
                forHeatFDCovAAX(here1,here2)=(fdCovAA1+fdCovAA2)/2;
                forHeatFDCovAA0(here1,here2)=fdCovAA0;
                
                %Use for Fig. S2.
                %This calculates corr(EP_x, EP_0) (i.e., the correlation
                %between cross-feeders and the cheater in how good an area
                %is for reproduciton.
                %Alt is the same, but correlated with the mean
                %cross-feeder.
                forHeatCorr(here1,here2)=(cor1+cor2)/2;
                forHeatCorrAlt(here1,here2)=corAlt;
            else
                
                %If everyone in the community died, then I put NaN in for
                %the value (and know later not to use that).
                here1=(ndist-1)/2+1;
                here2=(n2dist-1)/2+1;
                
                
                forHeatBirthX(here1,here2)=NaN;
                forHeatBirth0(here1,here2)=NaN;
                forHeatBirthXs(here1,here2)=NaN;
                forHeatBirth0s(here1,here2)=NaN;
                forHeatFDCovAAX(here1,here2)=NaN;
                forHeatFDCovAA0(here1,here2)=NaN;
                forHeatCorr(here1,here2)=NaN;
                forHeatCorrAlt(here1,here2)=NaN;
                
                theDisp(here1,here2)=theList(ii);
                theDiff(here1,here2)=theList2(jj);

                
            end
            
        end
    end
end

%bigGrid2=bigGrid(:,501:2500);
bigGrid2=flipud(bigGrid);

bigGridX=bigGrid2;
bigGridX([1:3:size(bigGridX,1)],:)=[];
bigGridX(:,[1:3:size(bigGridX,2)])=[];
bigGridX([1:2:size(bigGridX,1)],:)=[];
bigGridX(:,[1:2:size(bigGridX,2)])=[];


figA=figure();

spy(bigGridX==2,'g.',.0000001)
hold on
spy(bigGridX==3,'b.',.0000001)
spy(bigGridX==1,'m.',.0000001)
hold off

%title(['K = ', num2str(K), ', no max f(C)'])

set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel',[])

if(ANALYZE)
    save([thename,'_analyzed'])
end

set(figA,'Units','Inches');
pos = get(figA,'Position');
set(figA,'PaperPositionMode','Auto','PaperUnits','Inches','PaperSize',[pos(3), pos(4)])
saveas(figA,[figName, '.pdf'])


return

%The code below makes a solid swatch of each color, for the legend.
%
%Right now your code will not do this, but if you want it to, delete the
%'return' above.

bob=ones(100);
figB=figure();
spy(bob,'g.')
set(figB,'Units','Inches');
pos = get(figB,'Position');
set(figB,'PaperPositionMode','Auto','PaperUnits','Inches','PaperSize',[pos(3), pos(4)])
saveas(figB,['green.pdf'])

figB=figure();
spy(bob,'b.')
set(figB,'Units','Inches');
pos = get(figB,'Position');
set(figB,'PaperPositionMode','Auto','PaperUnits','Inches','PaperSize',[pos(3), pos(4)])
saveas(figB,['blue.pdf'])

figB=figure();
spy(bob,'m.')
set(figB,'Units','Inches');
pos = get(figB,'Position');
set(figB,'PaperPositionMode','Auto','PaperUnits','Inches','PaperSize',[pos(3), pos(4)])
saveas(figB,['magenta.pdf'])